﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DataLayer.ComLayers
{
    /// <summary>
    /// Base class for all communication layers. Contains basic data processing. The idea is to put all received bytes from
    /// any layer (serial, udp, bt, tcp/ip) to this class, where it would be processed in the same way (because it is expected
    /// always the same "raw data" regardless of transport layer)
    /// 
    /// NOTE: this could be (shall be?) moved to the Marvemind ModemFacade. But in case implementing different facades for 
    /// different HW (with the same communication schema), it shall remain be here.
    /// </summary>
    public class BaseLayer
    {
        #region Public Members

        public event PacketReceivedHandler OnPacketReceived;

        #endregion

        #region Private Members

        private List<byte> _recBuffer = new List<byte>();
        private byte _payloadLen = 0x00;

        #endregion

        #region Public Methods

        /// <summary>
        /// Process received bytes
        /// </summary>
        /// <param name="buffer"></param>
        public void ProcessBytes(byte[] buffer)
        {
            foreach (var b in buffer)
                ProcessByte(b);
        }

        /// <summary>
        /// Process one received byte
        /// </summary>
        /// <param name="b"></param>
        public void ProcessByte(byte b)
        {
            if (b == 0xFF && _recBuffer.Count == 0)
            {// communication start 
                _recBuffer.Add(b);            // start packet receiving by inserting starting symbol to the buffer
            }
            else if (_recBuffer.Count > 0)
            {// in the process of receiving packet
                _recBuffer.Add(b);

                if (_recBuffer.Count == 5)
                {// just received payload length
                    _payloadLen = b;
                }
                else if (_recBuffer.Count == _payloadLen + 7)
                {// end of transmission
                    OnPacketReceived?.Invoke(_recBuffer.ToArray());
                    ResetReceive();
                }
            }
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Resets any state variables for receiving
        /// </summary>
        private void ResetReceive()
        {
            _recBuffer.Clear();
            _payloadLen = 0x00;
        }

        #endregion
    }
}
