﻿using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Linq;
using System.Text;

namespace DataLayer.ComLayers
{
    /// <summary>
    /// Class to communicate with the server using virtual serial port (via usb)
    /// </summary>
    public class SerialLayer : BaseLayer, IDataLayer
    {
        #region Public Members

        /// <summary>
        /// Event on receiving complete packet
        /// </summary>
        public new event PacketReceivedHandler OnPacketReceived;

        #endregion

        #region Public Methods

        /// <summary>
        /// Starts communication
        /// </summary>
        public void Start()
        {
            _serialPort.Open();
        }

        /// <summary>
        /// Stops communication
        /// </summary>
        public void Stop()
        {
            if (_serialPort.IsOpen)
                _serialPort.Close();
        }

        #endregion
        public SerialLayer(string comPortName)
        {
            _serialPort = new SerialPort(comPortName);

            _serialPort.BaudRate = 9600;
            _serialPort.Parity = Parity.None;
            _serialPort.StopBits = StopBits.One;
            _serialPort.DataBits = 8;
            _serialPort.Handshake = Handshake.None;
            _serialPort.RtsEnable = true;
            _serialPort.DataReceived += new SerialDataReceivedEventHandler(DataReceived);
            base.OnPacketReceived += new PacketReceivedHandler(PacketReceivedInternal);
        }

        #region Private Members

        SerialPort _serialPort = null;

        #endregion

        #region Private Methods

        /// <summary>
        /// Internal handler for received data packet
        /// </summary>
        /// <param name="buffer"></param>
        private void PacketReceivedInternal(byte[] buffer)
        {
            this.OnPacketReceived?.Invoke(buffer);
        }

        /// <summary>
        /// Received serial data
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            // read all available data
            var sp = (SerialPort)sender;
            var buffer = new byte[sp.BytesToRead];
            sp.Read(buffer, 0, buffer.Length);

            // process all data
            base.ProcessBytes(buffer);
        }

        #endregion
    }
}
