﻿using DataLayer;
using DataLayer.ComLayers;
using DataLayer.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Scanner.Code;

namespace Scanner
{
    public partial class MainForm : Form
    {
        #region Private Memberes
        /// <summary>
        /// Display item for beacon data
        /// </summary>
        private class BeaconDI
        {
            [DisplayName("Device address")]
            public byte Address { get; set; }

            [DisplayName("X [mm]")]
            public Int32 X { get; set; }

            [DisplayName("Y [mm]")]
            public Int32 Y { get; set; }

            [DisplayName("Z [mm]")]
            public Int32 Z { get; set; }
        }

        private ModemFacade _modem = null;  // object to access marvelmind modem
        private BindingList<BeaconDI> _beaconDIs = new BindingList<BeaconDI>();


        #endregion

        public MainForm()
        {
            InitializeComponent();

            this.Text += $" (v.{Constants.Version})";
            

            // initialize grid for beacons data
            dgvData.DataSource = this._beaconDIs;
            foreach (DataGridViewColumn col in dgvData.Columns)
                col.AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;
            if (dgvData.Columns.Count > 0)
                dgvData.Columns[0].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;

            // initialize serial ports
            cbPorts.DataSource = SerialPort.GetPortNames();
        }

        #region Private Events

        /// <summary>
        /// Starts communication with modem
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOpen_Click(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(cbPorts.Text))
                {
                    MessageBox.Show("Serial port is not set", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                
                if (_modem != null)
                    _modem = null;

                _modem = new ModemFacade(new SerialLayer("COM4"));  // NOTE: when using different physical communicatin layer, use different communication layer object
                _modem.OnPacketReceive += _modem_OnPacketReceive;

                _modem.Connect();
                btnStop.Enabled = true;
                btnStart.Enabled = false;
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Stops communication with modem
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnClose_Click(object sender, EventArgs e)
        {
            try
            {
                _modem.Disconnect();
                btnStop.Enabled = false;
                btnStart.Enabled = true;
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Valid packet from modem has been received
        /// </summary>
        /// <param name="packet"></param>
        private void _modem_OnPacketReceive(Packet packet)
        {
            try
            {
                if (this.InvokeRequired)
                {
                    this.Invoke(new ModemFacade.PacketReceivedHandler(_modem_OnPacketReceive), new object[] { packet });
                    return;
                }

                if (packet.PacketType == 0x47 &&
                    packet.PacketCode == 0x11)
                {
                    var p47_11 = packet.CastPacket<Packet47_11>();

                    var di = _beaconDIs.FirstOrDefault(x => x.Address == p47_11.Address);
                    if (di == null)
                    {// beacon not added yet, add it now (set only address, coordinates will be updated)
                        di = new BeaconDI()
                        {
                            Address = p47_11.Address
                        };
                        _beaconDIs.Add(di);
                    }

                    di.X = p47_11.X;
                    di.Y = p47_11.Y;
                    di.Z = p47_11.Z;

                }
                else if (packet.PacketType == 0x47 &&
                         packet.PacketCode == 0x12)
                {
                    var p47_12 = packet.CastPacket<Packet47_12>();

                    foreach (var beacon in p47_12.Beacons)
                    {
                        var di = _beaconDIs.FirstOrDefault(x => x.Address == beacon.Address);
                        if (di == null)
                        {// beacon not added yet, add it now (set only address, coordinates will be updated)
                            di = new BeaconDI()
                            {
                                Address = beacon.Address
                            };
                            _beaconDIs.Add(di);
                        }

                        di.X = beacon.X;
                        di.Y = beacon.Y;
                        di.Z = beacon.Z;
                    }
                }
                else
                {
                    ; // 
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        #endregion
    }
}